
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;

/**
 * Class that output into a txtfile num many rows of three columns
 *
 * @author SJSU Students
 *
 */
public class DataGenerator {

	/**
	 * Main method that takes in user's input and generate the text file with data
	 *
	 * @param args - user's inputs
	 */
	public static void main(String[] args) {
		// expect 5 inputs
		if (args.length != 5) {
			System.out.println("Incorrect amount of parameter");
			printUsages();
		} else {
			// variables expecting
			int start_value = -1;
			int num = -1;
			int wrap_number1 = -1;
			int wrap_number2 = -1;
			String fileName = "";

			// check start_value
			if (!isNumeric(args[0])) {
				System.out.println("Invalid start value");
				return;
			}
			start_value = Integer.parseInt(args[0]);

			// check num
			if (!isNumeric(args[1])) {
				System.out.println("Invalid number of rows");
				return;
			}
			num = Integer.parseInt(args[1]);

			// check wrap_number1
			if (!isNumeric(args[2])) {
				System.out.println("Invalid wrap_number1");
				return;
			}
			wrap_number1 = Integer.parseInt(args[2]);

			// check wrap_number2
			if (!isNumeric(args[3])) {
				System.out.println("Invalid wrap_number2");
				return;
			}
			wrap_number2 = Integer.parseInt(args[3]);

			// check txtfile
			if (args[4].trim().isBlank()) {
				System.out.println("Invalid file");
				return;
			}
			fileName = args[4].trim();

			// generate data
			generateData(start_value, num, wrap_number1, wrap_number2, fileName);
		}
	}

	/**
	 * Method that checks if string is numeric
	 *
	 * @param s - String to check
	 * @return true if String is numeric; false if String is not numeric
	 */
	private static boolean isNumeric(String s) {
		try {
			Integer.parseInt(s);
			return true;
		} catch (NumberFormatException e) {
			return false;
		}
	}

	/**
	 * Method that generate a random number between 1 and wrap_number
	 *
	 * @param wrap_number - upper bound of randomization
	 * @return random integer between a range
	 */
	private static int randNumberGenerator(int wrap_number) {
		// generate random number using wrap_number....
		return (int) ((Math.random() * (wrap_number + 1 - 1)) + 1);
	}

	/**
	 * Method that write to textfile the generated data
	 *
	 * @param start_value  - for first column
	 * @param num          - number of rows
	 * @param wrap_number1 - upper bound for random number generator in column 2
	 * @param wrap_number2 - upper bound for random number generator in column 3
	 * @param fileName     - file to write into
	 */
	private static void generateData(int start_value, int num, int wrap_number1, int wrap_number2, String fileName) {
		// write data to fileName
		try {
			BufferedWriter bw = new BufferedWriter(new FileWriter(fileName));

			for (int i = 0; i < num; i++) {
				bw.write(start_value + " " + randNumberGenerator(wrap_number1) + " " + randNumberGenerator(wrap_number2)
						+ "\n");
				start_value++;
			}
			bw.close();
			System.out.println("Finished writing generated data into fileName " + fileName);
		} catch (IOException e) {
			e.printStackTrace();
		}

	}

	/**
	 * Method that prints out the usage
	 */
	private static void printUsages() {
		// expecting 5 inputs
		// start_value num wrap_number1 wrap_number2 txtfile
		System.out.println("Usage:");
		System.out.println("java DataGenerator start_value num wrap_number1 wrap_number2 txtfile");
		System.out.println("start_value - value of the first column. Will increment per row");
		System.out.println("num - number of rows of three columns");
		System.out.println("wrap_number1 - second column's value is chosen randomly from between 1 and wrap_number1");
		System.out.println("wrap_number2 - third column's value is chosen randomly from between 1 and wrap_number2");
		System.out.println("txtfile - output file ");
	}
}
